local addon, EZ = ...

local default_settings = {
	ctrlsize=48, locked = false,
	cfgx = 100, cfgy = 100,
	ctrlx=UIParent:GetWidth()/2, ctrly=UIParent:GetHeight()/2,
	roles = {} }

local SECURE_WARNING = "WARNING: Unable to modify state whilst in secure mode (combat). Try again later."

local _ezROLEID = 0

local _ezUIBUILT = false
local _ezSECURE = false
local _ezSCANNEEDED = false
local _ezBUFFMISSING = false

local _ezACTIVEBUFFID = {}
local _ezABILITYSCAN = {}
local _ezABILITYMISSING = {}

local _playerID = Inspect.Unit.Lookup("player")

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}				
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

function EZ.Event_Addon_SavedVariables_Load_End(h, a)
	if a == addon.identifier then
		if EZRebuff_Settings == nil then EZRebuff_Settings = {} end
		MergeTable(EZRebuff_Settings, default_settings)
		EZ.UI.config:SetPoint("TOPLEFT", UIParent, "TOPLEFT", EZRebuff_Settings.cfgx, EZRebuff_Settings.cfgy)
		_ezSECURE = true
		LibVersionCheck.register(addon.toc.Identifier, addon.toc.Version)
	end
end

function EZ.SetupConfig()
	local ridx = string.format("h%016d", _ezROLEID-1)
	local rdtl = Inspect.Role.List()[ridx]
	EZ.UI.config_hdr:SetText(string.format("#%d [%s]", _ezROLEID, rdtl))
	if EZRebuff_Settings.roles[_ezROLEID] == nil then
		EZRebuff_Settings.roles[_ezROLEID] = {}
	end
	local ab = Inspect.Ability.New.List()
	
	local blist = {}
	
	for k,v in pairs(Inspect.Buff.Detail("player", Inspect.Buff.List("player"))) do
		if v.caster == _playerID and v.abilityNew and ab[v.abilityNew] then
			table.insert(blist, {n=v.name, i=v.icon, a=v.abilityNew})
		end
	end
	
	table.sort(blist, function(a,b) return a.n < b.n end)
	local idx = #blist
	
	for x=1, 16 do
		EZ.UI.config_data[x].c:SetVisible(x <= idx)
		if x <= idx then
			EZ.UI.config_data[x].i:SetTexture("Rift", blist[x].i)
			EZ.UI.config_data[x].t:SetText(blist[x].n)
			EZ.UI.config_data[x].c.IDNAME = blist[x].a
			EZ.UI.config_data[x].c.ICON = blist[x].i
			if EZRebuff_Settings.roles[_ezROLEID][blist[x].a] then
				EZ.UI.config_data[x].c:SetChecked(true)
			else
				EZ.UI.config_data[x].c:SetChecked(false)
			end
		else
			EZ.UI.config_data[x].c:SetChecked(false)
		end
	end
end

function EZ.SecureSetup()
	if Inspect.System.Secure() == false then
		EZ.UI.control:SetPoint("TOPLEFT", UIParent, "TOPLEFT", EZRebuff_Settings.ctrlx, EZRebuff_Settings.ctrly)
		EZ.UI.control:SetWidth(EZRebuff_Settings.ctrlsize)
		EZ.UI.control:SetHeight(EZRebuff_Settings.ctrlsize)
		local hs = math.floor(EZRebuff_Settings.ctrlsize/2)
		for x=1, 5 do
			EZ.UI.nextbuff[x]:SetWidth(hs)
			EZ.UI.nextbuff[x]:SetHeight(hs)
		end
		EZ.UI.control:SetSecureMode("restricted")
		if EZRebuff_Settings.locked then
			if _ezBUFFMISSING then
				local bd = {}
				for k,v in pairs(Inspect.Ability.New.Detail(_ezABILITYMISSING)) do
					table.insert(bd, {n=v.name, i=v.icon})
					table.sort(bd, function(a,b) return a.n < b.n end)
				end
				
				for x=1, 5 do
					if x <= #bd then
						EZ.UI.nextbuff[x]:SetTexture("Rift", bd[x].i)
					end
					EZ.UI.nextbuff[x]:SetVisible(x <= #bd)
				end
				
				EZ.UI.control:SetTexture(addon.identifier, "img/brain_red.png")
				EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Back, "cast "..bd[1].n)
				EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Forward, "cast "..bd[1].n)
			else
				EZ.UI.nextbuff[1]:SetVisible(false)
				EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Back, nil)
				EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Forward, nil)
				EZ.UI.control:SetTexture(addon.identifier, "img/brain_green.png")
			end
		else
			EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Back, nil)
			EZ.UI.control:EventMacroSet(Event.UI.Input.Mouse.Wheel.Forward, nil)
			EZ.UI.control:SetTexture(addon.identifier, "img/brain_gray.png")
		end
		EZ.UI.control:SetVisible(true)
		_ezSECURE = false
		_ezUIBUILT = true
	else
		_ezSECURE = true
	end
end

function EZ.BuildUI()
	EZ.UI = {}
	EZ.UI.ctx = UI.CreateContext(addon.identifier)
	EZ.UI.secctx = UI.CreateContext(addon.identifier)
	EZ.UI.secctx:SetSecureMode("restricted")
	
	EZ.UI.control = UI.CreateFrame("Texture", "EZ.UI.control", EZ.UI.secctx)
	EZ.UI.control:SetVisible(false)

	EZ.UI.nextbuff = {}
	
	for x=1,5 do
		local i = UI.CreateFrame("Texture", "nextbuff:"..x, EZ.UI.control)
		EZ.UI.nextbuff[x] = i
	end
	
	EZ.UI.nextbuff[3]:SetPoint("BOTTOMRIGHT", EZ.UI.control, "TOPLEFT", -2, -2)
	EZ.UI.nextbuff[2]:SetPoint("TOPLEFT", EZ.UI.nextbuff[3], "TOPRIGHT", 2, 0)
	EZ.UI.nextbuff[1]:SetPoint("TOPLEFT", EZ.UI.nextbuff[2], "TOPRIGHT", 2, 0)
	EZ.UI.nextbuff[4]:SetPoint("TOPLEFT", EZ.UI.nextbuff[3], "BOTTOMLEFT", 0, 2)
	EZ.UI.nextbuff[5]:SetPoint("TOPLEFT", EZ.UI.nextbuff[4], "BOTTOMLEFT", 0, 2)
	
	EZ.UI.config = UI.CreateFrame("RiftWindow", "EZ.UI.config", EZ.UI.ctx)
	EZ.UI.config:SetVisible(false)
	EZ.UI.config:SetWidth(256)
	EZ.UI.config:SetHeight(512)
	EZ.UI.config:SetTitle("EZRebuff Configuration")

	EZ.UI.config_close = UI.CreateFrame("RiftButton", "EZ.UI.config_close", EZ.UI.config)
	EZ.UI.config_close:SetSkin("close")
	EZ.UI.config_close:SetPoint("TOPRIGHT", EZ.UI.config, "TOPRIGHT", -6,6)
	EZ.UI.config_close:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		EZ.UI.config:SetVisible(false)
	end, "Event.UI.Input.Mouse.Left.Click")

	local trl, trt, trr, trb
	trl, trt, trr, trb = EZ.UI.config:GetTrimDimensions()
	
	EZ.UI.config_apply = UI.CreateFrame("RiftButton", "EZ.UI.config_close", EZ.UI.config)
	EZ.UI.config_apply:SetPoint("CENTERBOTTOM", EZ.UI.config, "CENTERBOTTOM", 0, -(trb+4))
	EZ.UI.config_apply:SetText("Apply")
	
	EZ.UI.config_apply:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		EZRebuff_Settings.roles[_ezROLEID] = {}
		for x=1, 16 do
			if EZ.UI.config_data[x].c:GetChecked() then
				EZRebuff_Settings.roles[_ezROLEID][EZ.UI.config_data[x].c.IDNAME] = EZ.UI.config_data[x].c.ICON
			end
		end
		EZ.ClearBuffs()
		EZ.ScanBuffs()
	end, "Event.UI.Input.Mouse.Left.Click")
	
	EZ.UI.config_hdr = UI.CreateFrame("Text", "EZ.UI.config_hdr", EZ.UI.config)
	EZ.UI.config_hdr:SetPoint("TOPLEFT", EZ.UI.config, "TOPLEFT", trl, trt)
	EZ.UI.config_hdr:SetFontSize(14)
	
	EZ.UI.config_data = {}
	
	for x=1,16 do
		local c = UI.CreateFrame("RiftCheckbox", "checkbox:"..x, EZ.UI.config)
		c:SetHeight(24)
		c:SetWidth(24)
		c:SetVisible(false)
		local i = UI.CreateFrame("Texture", "texture:"..x, c)
		i:SetHeight(24)
		i:SetWidth(24)
		i:SetPoint("CENTERLEFT", c, "CENTERRIGHT", 2,0)
		local t = UI.CreateFrame("Text", "text:"..x, c)
		t:SetFontSize(12)
		t:SetPoint("CENTERLEFT", i, "CENTERRIGHT", 2,0)
		t:SetText("DATA ROW#"..x)
		if x==1 then
			c:SetPoint("TOPLEFT", EZ.UI.config_hdr, "BOTTOMLEFT", 0,0)
		else
			c:SetPoint("TOPLEFT", EZ.UI.config_data[x-1].c, "BOTTOMLEFT", 0,0)
		end
		EZ.UI.config_data[x] = {c=c, i=i, t=t}
	end
	
	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if EZRebuff_Settings.locked == false then
			if Inspect.System.Secure() == false then
				local mouseData = Inspect.Mouse()
				self.sx = mouseData.x - self:GetLeft()
				self.sy = mouseData.y - self:GetTop()
				self.MouseDown = true
			else
				print(SECURE_WARNING)				
			end
		end
	end, "Event.UI.Input.Mouse.Left.Down")

	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if Inspect.System.Secure() == false then
			if self.MouseDown then
				local nx, ny
				local mouseData = Inspect.Mouse()
				nx = mouseData.x - self.sx
				ny = mouseData.y - self.sy
				self:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
			end
		else
			self.MouseDown = false
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Inspect.System.Secure() == false then
			if EZRebuff_Settings.locked == false then
				if EZRebuff_Settings.ctrlsize < 64 then
					EZRebuff_Settings.ctrlsize = EZRebuff_Settings.ctrlsize+1
					EZ.SecureSetup()
				end
			end
		else
			print(SECURE_WARNING)				
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")

	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Inspect.System.Secure() == false then
			if EZRebuff_Settings.locked == false then
				if EZRebuff_Settings.ctrlsize > 8 then
					EZRebuff_Settings.ctrlsize = EZRebuff_Settings.ctrlsize-1
					EZ.SecureSetup()
				end
			end
		else
			print(SECURE_WARNING)				
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")
	
	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		EZRebuff_Settings.ctrlx = self:GetLeft()
		EZRebuff_Settings.ctrly = self:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")
	
	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		if Inspect.System.Secure() == false then
			EZRebuff_Settings.locked = not EZRebuff_Settings.locked
			EZ.SecureSetup()
		else
			print(SECURE_WARNING)
		end
	end, "Event.UI.Input.Mouse.Right.Click")
	
	EZ.UI.control:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if EZRebuff_Settings.locked then
			local vis = not EZ.UI.config:GetVisible()
			EZ.UI.config:SetVisible(vis)
			if vis then
				EZ.SetupConfig()
			end
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	local cfgborder = EZ.UI.config:GetBorder()
	
	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		local mouseData = Inspect.Mouse()
		self.sx = mouseData.x - EZ.UI.config:GetLeft()
		self.sy = mouseData.y - EZ.UI.config:GetTop()
		self.MouseDown = true
	end, "Event.UI.Input.Mouse.Left.Down")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			EZ.UI.config:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	cfgborder:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		self.MouseDown = false
		EZRebuff_Settings.cfgx = EZ.UI.config:GetLeft()
		EZRebuff_Settings.cfgy = EZ.UI.config:GetTop()
	end, "Event.UI.Input.Mouse.Left.Up")
end

function EZ.Command_Slash_Register(h,args)
	local r = {}
	for token in string.gmatch(args, "[^%s]+") do
		table.insert(r, token)
	end
end

function EZ.Event_System_Secure_Enter(h)
	EZ.UI.ctx:SetVisible(false)
	EZ.UI.secctx:SetVisible(false)
	EZ.UI.control.mouseDown = false
end

function EZ.Event_System_Secure_Leave(h)
	EZ.UI.ctx:SetVisible(true)
	EZ.UI.secctx:SetVisible(true)
end

function EZ.Event_System_Update_Begin(h)
	if _ezSECURE then
		EZ.SecureSetup()
	end
	if _ezSCANNEEDED then
		EZ.ScanBuffs()
	end
end

function EZ.ClearBuffs()
	_ezACTIVEBUFFID = {}
	_ezABILITYSCAN = EZRebuff_Settings.roles[_ezROLEID]
	_ezABILITYMISSING = {}	
	for k,v in pairs(_ezABILITYSCAN) do
		_ezABILITYMISSING[k] = v
	end
	_ezSCANNEEDED = true
end

function EZ.ScanBuffs()

	local ab = Inspect.Ability.New.List()
	if ab then
		for k,v in pairs(_ezABILITYSCAN) do
			if ab[k] then
				_ezABILITYMISSING[k] = v
			end
		end
		for k,v in pairs(Inspect.Buff.Detail("player", Inspect.Buff.List("player"))) do
			if v.caster == _playerID and v.abilityNew and _ezABILITYSCAN[v.abilityNew] then
				_ezABILITYMISSING[v.abilityNew] = nil
				_ezACTIVEBUFFID[k] = true
			end
		end
		if next(_ezABILITYMISSING) then
			_ezBUFFMISSING = true
		else
			_ezBUFFMISSING = false
		end
		EZ.SecureSetup()
		_ezSCANNEEDED = false
	else
		_ezSCANNEEDED = true
	end
end

function EZ.Event_TEMPORARY_Role(h, r)
	if r > 0 and r ~= _ezROLEID then
		_ezROLEID = r
		if EZRebuff_Settings.roles[_ezROLEID] == nil then
			EZRebuff_Settings.roles[_ezROLEID] = {}
		end		
		EZ.ClearBuffs()
	end
end

function EZ.Event_Buff_Add(h,u,t)
	if u == _playerID then
		if EZ.UI.config:GetVisible() then
			EZ.SetupConfig()
		end
		_ezSCANNEEDED = true
	end
end

function EZ.Event_Buff_Remove(h,u,t)
	if u == _playerID then
		for k,v in pairs(t) do
			if _ezACTIVEBUFFID[k] then
				_ezSCANNEEDED = true
				break
			end
		end
	end
end

function EZ.Event_Unit_Availability_Full(h,t)
	for k,v in pairs(t) do
		if v == "player" then
			EZ.Event_TEMPORARY_Role(h, Inspect.TEMPORARY.Role())
			Command.Event.Detach(Event.Unit.Availability.Full, nil, nil, nil, addon.identifier)
		end
	end
end

function EZ.Event_Ability_New_Add(h, t)
	for k,v in pairs(t) do
		if _ezABILITYSCAN[k] then
			_ezSCANNEEDED = true
			break
		end
	end
end

EZ.BuildUI()

Command.Event.Attach(Event.Addon.SavedVariables.Load.End, EZ.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End" )
Command.Event.Attach(Command.Slash.Register("ezrebuff"), EZ.Command_Slash_Register, "Command.Slash.Register")
Command.Event.Attach(Event.System.Secure.Enter, EZ.Event_System_Secure_Enter, "Event.System.Secure.Enter")
Command.Event.Attach(Event.System.Secure.Leave, EZ.Event_System_Secure_Leave, "Event.System.Secure.Leave")
Command.Event.Attach(Event.System.Update.Begin, EZ.Event_System_Update_Begin, "Event.System.Update.Begin")
Command.Event.Attach(Event.TEMPORARY.Role, EZ.Event_TEMPORARY_Role, "Event.TEMPORARY.Role")
Command.Event.Attach(Event.Buff.Add, EZ.Event_Buff_Add, "Event.Buff.Add")
Command.Event.Attach(Event.Buff.Remove, EZ.Event_Buff_Remove, "Event.Buff.Remove")
Command.Event.Attach(Event.Unit.Availability.Full, EZ.Event_Unit_Availability_Full, "Event.Unit.Availability.Full")
Command.Event.Attach(Event.Ability.New.Add, EZ.Event_Ability_New_Add, "Event.Ability.New.Add")
Command.Event.Attach(Event.Ability.New.Remove, EZ.Event_Ability_New_Add, "Event.Ability.New.Remove")

print(string.format("v%s loaded.", addon.toc.Version))